/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */
#ifndef BENCHMARK_H_
#define BENCHMARK_H_

#include <vector>
#include <string>
#include <map>

#include "composite-test.h"

class Benchmark
{
public:
    typedef std::pair<std::string, std::string> OptionPair;

    Benchmark(CompositeTest &test, const std::vector<OptionPair> &options);
    Benchmark(const std::string &name, const std::vector<OptionPair> &options);
    // Create a benchmark from a description string of the form:
    // test[:opt1=val1:opt2=val2...]
    Benchmark(const std::string &s);

    CompositeTest &setup_test();
    void teardown_test();

    static void register_test(CompositeTest &test);
    static CompositeTest &get_test_by_name(const std::string &name);
    static const std::map<std::string, CompositeTest *> &tests() { return test_map_; }

private:
    CompositeTest &test_;
    std::vector<OptionPair> options_;

    void load_options();

    static std::map<std::string, CompositeTest *> test_map_;
};

#endif
