/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#ifndef COMPOSITE_CANVAS_H_
#define COMPOSITE_CANVAS_H_

#include <stdint.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <list>
#include <vector>

#include "composite-window.h"
#include "composite-test.h"
#include "benchmark.h"
#include "profiler.h"

class CompositeCanvas
{
public:
    CompositeCanvas():
        xdpy_(0), root_(0), canvas_(0), damage_event_(0),
        current_test_(0), iterations_(0), width_(0), height_(0),
        profiler_cycle_pair_("cycle"),
        profiler_xevent_pair_("xevent"),
        profiler_draw_pair_("draw"),
        profiler_screen_update_pair_("screen-update")
    {}

    bool init();
    void run_tests(std::list<Benchmark*> &benchmarks);
    virtual std::string info_string() { return std::string(); }

protected:
    virtual XVisualInfo *get_canvas_xvisualinfo() = 0;
    virtual bool make_current() = 0;
    virtual CompositeWindow *create_window(Window win) = 0;
    virtual void update_screen() = 0;

    Display *xdpy_;
    Window root_;
    Window canvas_;
    int damage_event_;

    std::list<CompositeWindow*> window_list_;
    CompositeTest *current_test_;

private:
    CompositeWindow *find_window(Window win);
    void add_window(Window win);
    void remove_window(Window win);
    void update_window(Window win);
    void reshape(int width, int height);
    Window create_canvas_x_window(const char *name, int x, int y,
                                  int width, int height,
                                  XVisualInfo *vis_info);
    bool check_extensions();
    bool redirect_windows();
    void handle_damage(Window win);
    bool handle_xevent();
    uint64_t benchmark_timestamp();
    void benchmark_init();
    void benchmark_update(bool& iteration_complete);
    void update_all_textures();
    void sample_point(int i);
    void load_current_test_options();
    unsigned int iterations_;
    unsigned int width_;
    unsigned int height_;
    double current_test_duration_;
    unsigned int current_test_iterations_;

    Profiler::PointPair profiler_cycle_pair_;
    Profiler::PointPair profiler_xevent_pair_;
    Profiler::PointPair profiler_draw_pair_;
    Profiler::PointPair profiler_screen_update_pair_;
};


#endif /* COMPOSITE_CANVAS_H_ */
