/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#include "composite-test.h"
#include "options.h"
#include "log.h"

using std::string;
using std::map;

CompositeTest::CompositeTest(const string& test_name) :
    name_(test_name)
{
    options_["duration"] = CompositeTest::Option("duration", "3.0",
                                                 "The duration of each iteration in seconds");
    options_["iterations"] = CompositeTest::Option("iterations", "5",
                                                   "The number of iterations run for this test");
}

/**
 * Gets the number of window that are visible.
 *
 * @param window_list the window list to search in
 *
 * @return the number of visible windows
 */
int
CompositeTest::num_visible_windows(std::list<CompositeWindow *> &window_list)
{
    int count = 0;

    for(std::list<CompositeWindow*>::iterator iter = window_list.begin();
        iter != window_list.end(); iter++)
    {
        if ((*iter)->get_texture())
            count++;
    }

    return count;
}

bool
CompositeTest::set_option(const string &opt, const string &val)
{ 
    map<string, Option>::iterator iter = options_.find(opt);

    if (iter == options_.end())
        return false;

    iter->second.value = val;

    return true;
}

void
CompositeTest::reset_options()
{
    for (map<string, Option>::iterator iter = options_.begin();
         iter != options_.end();
         iter++)
    {
        Option &opt = iter->second;

        opt.value = opt.default_value;
    }
}

bool
CompositeTest::set_option_default(const string &opt, const string &val)
{
    map<string, Option>::iterator iter = options_.find(opt);

    if (iter == options_.end())
        return false;

    iter->second.default_value = val;

    return true;
}
