/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#include <X11/Xlib.h>

#include "gl-headers.h"

#include <EGL/egl.h>
#include <EGL/eglext.h>

#include "composite-window-eglimage.h"
#include "log.h"

extern PFNEGLCREATEIMAGEKHRPROC eglCreateImageKHR_;
extern PFNEGLDESTROYIMAGEKHRPROC eglDestroyImageKHR_;
extern PFNGLEGLIMAGETARGETTEXTURE2DOESPROC glEGLImageTargetTexture2DOES_;

CompositeWindowEGLImage::~CompositeWindowEGLImage()
{
    if (egl_image_)
        eglDestroyImageKHR_(egl_display_, egl_image_);
}

void
CompositeWindowEGLImage::update_texture_from_pixmap(bool recreate)
{
    const EGLint egl_image_attribs[] = {
        EGL_IMAGE_PRESERVED_KHR, EGL_TRUE,
        EGL_NONE
    };

    Profiler::ScopedSamples scoped_tfp(CompositeWindow::get_profiler_tfp_pair());

    if (recreate) {
        if (egl_image_) {
            eglDestroyImageKHR_(egl_display_, egl_image_);
            egl_image_ = 0;
        }
        egl_image_ = eglCreateImageKHR_(egl_display_, EGL_NO_CONTEXT,
                                        EGL_NATIVE_PIXMAP_KHR,
                                        reinterpret_cast<EGLClientBuffer>(pix_),
                                        egl_image_attribs);

        Log::debug("    => EGLImage <= %p\n", egl_image_);
        Log::debug("    => eglError 0x%x\n", eglGetError());
    }

    /* Update texture with new data */
    eglWaitNative(EGL_CORE_NATIVE_ENGINE);

    glBindTexture(GL_TEXTURE_2D, tex_);

    glEGLImageTargetTexture2DOES_(GL_TEXTURE_2D,
                                  static_cast<GLeglImageOES>(egl_image_));
}
