/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#include <X11/Xlib.h>

#define GLX_GLXEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glx.h>
#include <GL/glxext.h>

#include "composite-window-glxpixmap.h"
#include "log.h"

extern PFNGLXBINDTEXIMAGEEXTPROC glXBindTexImageEXT_;

CompositeWindowGLXPixmap::~CompositeWindowGLXPixmap()
{
    if (glx_pixmap_)
        glXDestroyPixmap(xdpy_, glx_pixmap_);
}

void
CompositeWindowGLXPixmap::update_texture_from_pixmap(bool recreate)
{
    const int pixmap_attribs[] = {
        GLX_TEXTURE_TARGET_EXT, GLX_TEXTURE_2D_EXT,
        GLX_TEXTURE_FORMAT_EXT, GLX_TEXTURE_FORMAT_RGBA_EXT,
        None
    };

    Profiler::ScopedSamples scoped_tfp(CompositeWindow::get_profiler_tfp_pair());

    if (recreate) {
        if (glx_pixmap_) {
            glXDestroyPixmap(xdpy_, glx_pixmap_);
            glx_pixmap_ = 0;
        }
        glx_pixmap_ = glXCreatePixmap(xdpy_, glx_fbconfig_, pix_,
                                      pixmap_attribs);

        Log::debug("    => GLXPixmap <= %p\n", glx_pixmap_);
    }

    /* Update texture with new data */
    glXWaitX();

    glBindTexture(GL_TEXTURE_2D, tex_);

    glXBindTexImageEXT_(xdpy_, glx_pixmap_, GLX_FRONT_LEFT_EXT, NULL);
}
