/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "gl-headers.h"

#include "composite-window-ximage.h"
#include "log.h"

CompositeWindowXImage::~CompositeWindowXImage()
{
    if (ximage_)
        XDestroyImage(ximage_);
}

void
CompositeWindowXImage::update_texture_from_pixmap(bool recreate)
{
    Profiler::ScopedSamples scoped_tfp(CompositeWindow::get_profiler_tfp_pair());

    glBindTexture(GL_TEXTURE_2D, tex_);

    if (recreate) {
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                width_, height_, 0,
                GL_RGBA, GL_UNSIGNED_BYTE, NULL);
    }

    if (ximage_)
        XDestroyImage(ximage_);

    ximage_ = XGetImage(xdpy_, pix_, 0, 0, width_, height_, AllPlanes, ZPixmap);
    if (ximage_) {
        /*
         * Note that the data we get from the pixmap is BGRA but the texture
         * expects RGBA, so what we are doing here is technically incorrect
         * (and the colours will not be correct).
         */
        glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0,
                        width_, height_,
                        GL_RGBA, GL_UNSIGNED_BYTE,
                        ximage_->data);
    }
    else {
        Log::debug("Ximage is NULL!\n");
    }
}
