/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#ifndef COMPOSITE_WINDOW_H_
#define COMPOSITE_WINDOW_H_

#include <X11/Xlib.h>
#include <X11/extensions/Xdamage.h>
#include "gl-headers.h"
#include "profiler.h"

class CompositeWindow
{
public:
    CompositeWindow(Display *xdpy, Window win) :
        xdpy_(xdpy), win_(win), pix_(0), damage_(0),
        tex_(0), width_(0), height_(0), depth_(0) {}

    virtual ~CompositeWindow();
    void update();
    void handle_damage();
    Window get_xwindow();
    GLuint get_texture();

    virtual void update_texture_from_pixmap(bool recreate) = 0;

protected:
    Display *xdpy_;
    Window win_;
    Pixmap pix_;
    Damage damage_;
    XWindowAttributes attr_;
    GLuint tex_;

    unsigned int width_;
    unsigned int height_;
    unsigned int depth_;

    static Profiler::PointPair &get_profiler_tfp_pair()
    {
        static Profiler::PointPair profiler_tfp_pair_("update-texture-from-pixmap");
        return profiler_tfp_pair_;
    }
};

#endif /* COMPOSITE_WINDOW_H_ */
